@@ -26,7 +26,7 @@
 #include "rlm_se.h"
 #include "psi_fns.h"
 #include "medianpolish.h"
-
+#include "common.h"
 
 
 
@@ -37,7 +37,6 @@
 #include <limits.h>
 #include <unistd.h>
 #define THREADS_ENV_VAR "R_THREADS"
-pthread_mutex_t mutex_R;
 struct loop_data{
   double *matrix;
   SEXP *R_return_value;
@@ -51,6 +50,16 @@ struct loop_data{
   int start_row;
   int end_row;
 };
+
+#ifdef __linux__
+#include <features.h>
+#ifdef __GLIBC__
+#ifdef __GLIBC_PREREQ && __GLIBC_PREREQ(2, 15)
+#define INFER_MIN_STACKSIZE 1
+#endif
+#endif
+#endif
+
 #endif
 
 
@@ -165,11 +174,17 @@ SEXP R_sub_rcModelSummarize_medianpolish(SEXP RMatrix, SEXP R_rowIndexList){
   double chunk_size_d, chunk_tot_d;
   char *nthreads;
   pthread_attr_t attr;
+  /* Initialize thread attribute */
+  pthread_attr_init(&attr);
   pthread_t *threads;
   struct loop_data *args;
   void *status; 
 #ifdef PTHREAD_STACK_MIN
-  size_t stacksize = PTHREAD_STACK_MIN + 0x4000;
+#ifdef INFER_MIN_STACKSIZE
+  size_t stacksize = __pthread_get_minstack(&attr) + sysconf(_SC_PAGE_SIZE);
+#else
+  size_t stacksize = PTHREAD_STACK_MIN + sysconf(_SC_PAGE_SIZE);
+#endif
 #else
   size_t stacksize = 0x8000;
 #endif
@@ -211,8 +226,7 @@ SEXP R_sub_rcModelSummarize_medianpolish(SEXP RMatrix, SEXP R_rowIndexList){
   }
   threads = (pthread_t *) Calloc(num_threads, pthread_t);
 
-  /* Initialize and set thread detached attribute */
-  pthread_attr_init(&attr);
+  /* Set thread detached attribute */
   pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
   pthread_attr_setstacksize (&attr, stacksize);
   
@@ -480,11 +494,17 @@ SEXP R_sub_rcModelSummarize_plm(SEXP RMatrix, SEXP R_rowIndexList, SEXP PsiCode,
   double chunk_size_d, chunk_tot_d;
   char *nthreads;
   pthread_attr_t attr;
+  /* Initialize thread attribute */
+  pthread_attr_init(&attr);
   pthread_t *threads;
   struct loop_data *args;
   void *status; 
 #ifdef PTHREAD_STACK_MIN
-  size_t stacksize = PTHREAD_STACK_MIN + 0x4000;
+#ifdef INFER_MIN_STACKSIZE
+  size_t stacksize = __pthread_get_minstack(&attr) + sysconf(_SC_PAGE_SIZE);
+#else
+  size_t stacksize = PTHREAD_STACK_MIN + sysconf(_SC_PAGE_SIZE);
+#endif
 #else
   size_t stacksize = 0x8000;
 #endif
@@ -532,8 +552,7 @@ SEXP R_sub_rcModelSummarize_plm(SEXP RMatrix, SEXP R_rowIndexList, SEXP PsiCode,
   }
   threads = (pthread_t *) Calloc(num_threads, pthread_t);
 
-  /* Initialize and set thread detached attribute */
-  pthread_attr_init(&attr);
+  /* Set thread detached attribute */
   pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
   pthread_attr_setstacksize (&attr, stacksize);
   
